# This script requires the IPIP120.por file available here: https://osf.io/wxvth/
# It also requires the IPIP-NEO-ItemKey.xls file  available here: https://osf.io/ycvdk/
# Open the latter in Excel and save it as a csv file with the filename IPIP-NEO-ItemKey.csv.
#
# Johnson, J. A. (2014). Measuring thirty facets of the five factor model with a 120-item 
# public domain inventory: Development of the IPIP-NEO-120. J. of Research in Personality, 
# 51, 78-89.

# Load libraries and functions and set sampling parameters:
set.seed(6199)
library(foreign)
n.min <- 25
n.max <- 100
agecut1 <- 18
agecut2 <- 21

# Load Johnson data:
mydat <- read.spss("IPIP120.por", to.data.frame=TRUE)
key <- read.csv("IPIP-NEO-ItemKey.csv", header=TRUE)

# Remove observations with missing data:
sel <- rowSums(mydat[,-(1:10)]==0) == 0
mydat <- mydat[sel,]

# Obtain score for each trait:
items <- as.matrix(mydat[,-(1:10)])
key <- key[1:120,]
five.factors <- matrix(NA, nrow(mydat), 5)
tmp <- c("N", "E", "O", "A", "C")
colnames(five.factors) <- tmp
for(i in 1:length(tmp)){
	sel <- grepl(tmp[i], key[,"Key"])
	five.factors[,i] <- rowMeans(items[,sel])
}

# Determine which countries have sufficient data for each sex / age group pair:
sel <- mydat$AGE>=agecut1
mydat <- mydat[sel,]
five.factors <- five.factors[sel,]
mydat$Adult <- mydat$AGE > agecut2
tmp <- table(mydat$COUNTRY, mydat$SEX, mydat$Adult)
tmp2 <- apply(tmp, 1, function(x){all(x>n.min)}) 
countries <- names(tmp2[tmp2==TRUE])

# Sample / subsample data from countries with sufficient data:
idx <- NULL
for(i in 1:length(countries)){
	for(j in c(TRUE,FALSE)){
		for(k in 1:2){
			tmp.idx <- which(mydat$COUNTRY==countries[i] & mydat$Adult==j & mydat$SEX==k)
			if(length(tmp.idx) > n.max){ tmp.idx <- sample(tmp.idx,n.max,replace=FALSE) }
			idx <- c(idx, tmp.idx)
		}
	}	
}

# Create paper data
dat <- data.frame(Country=as.character(mydat$COUNTRY[idx]),
	Male=mydat$SEX[idx]==1, Adult=mydat$Adult[idx], round(five.factors[idx,],2))
tmp <- table(dat$Country, dat$Male, dat$Adult)
o <- order(dat$Country, dat$Male, dat$Adult)
dat <- dat[o,]
write.csv(dat, file="study.data.csv", row.names=FALSE, quote=FALSE)
