# This script relies on:
#	(i)  study.data.csv which can be created by the step1.create.data.R script.
#	(ii) country.latlong.csv which was obtained from https://www.latlong.net
#		 after inputting the capital city of each country.
# These two files are included these supplementary materials.


###########################
# Load libraries and data #
###########################
library(lme4)
library(metafor)
library(reshape2)
library(Matrix)
library(nlme)
library(optimx)
matrixlist.to.matrix <- function(matrixlist){
	G <- length(matrixlist)
	M <- bdiag(matrixlist[[1]])
	for(g in 2:G){ M <- bdiag(M,matrixlist[[g]]) }
	M <- as.matrix(M)
	M
}
dat <- read.csv("study.data.csv", header=TRUE)
latlong <- read.csv("country.latlong.csv", header=TRUE)


dat$Group <- NA
dat$Group[dat$Male==FALSE & dat$Adult==FALSE] <- c("College; Female")
dat$Group[dat$Male==FALSE & dat$Adult==TRUE] <- c("Adult; Female")
dat$Group[dat$Male==TRUE & dat$Adult==FALSE] <- c("College; Male")
dat$Group[dat$Male==TRUE & dat$Adult==TRUE] <- c("Adult; Male")
dat$CountryGroup <- paste0(dat$Country, ": ", dat$Group)


countries <- as.character(latlong[,1])
abs.lat <- abs(latlong[,3])
names(abs.lat) <- countries
dat$abs.lat <- abs.lat[as.character(dat$Country)]


##########
# Part I #
##########
sel <- dat$Group=="Adult; Male"
m <- as.vector(by(dat$E[sel], dat$CountryGroup[sel], mean))
s <- as.vector(by(dat$E[sel], dat$CountryGroup[sel], sd))
n <- as.vector(by(dat$E[sel], dat$CountryGroup[sel], length))
country <- 1:length(m)

case1.indiv <- lmer(E~1+(1|Country), data=dat[sel,])
case1.summ <- rma.mv(m, s^2/n, random=~1|country)
rmse <- as.data.frame(VarCorr(case1.indiv))[2,5]
case1.summ2 <- rma.mv(m, rmse^2/n, random=~1|country)

summary(case1.indiv)
summary(case1.summ)
summary(case1.summ2)


###########
# Part II #
###########
sel <- dat$Group=="Adult; Male" | dat$Group=="Adult; Female"
m <- as.vector(by(dat$E[sel], dat$CountryGroup[sel], mean))
s <- as.vector(by(dat$E[sel], dat$CountryGroup[sel], sd))
n <- as.vector(by(dat$E[sel], dat$CountryGroup[sel], length))
group <- rep(c("Adult; Female", "Adult; Male"), length(countries))
country <- rep(countries, each=2)
country.group <- 1:length(m)

case2.indiv <- lmer(E~Group-1+(1|Country)+(1|CountryGroup), data=dat[sel,])
case2.summ <- rma.mv(m, s^2/n, mods=~group-1, random=list(~1|country,~1|country.group))

summary(case2.indiv)
summary(case2.summ)


idxF <- seq(1, length(m), by=2)
idxM <- idxF + 1
m.diff <- m[idxM] - m[idxF]
v <- s[idxM]^2/n[idxM] + s[idxF]^2/n[idxF]
country.diff <- 1:length(m.diff)
case2.summ.raw.diff <- rma.mv(m.diff, v, random=~1|country.diff)

v.pool <- ( (n[idxF]-1)*s[idxF]^2 + (n[idxM]-1)*s[idxM]^2 ) / (n[idxF]-1 + n[idxM]-1)
s.pool <- sqrt(v.pool)
d <- m.diff / s.pool
v.d <- 1/n[idxM] + 1/n[idxF]
case2.summ.cohen.d <- rma.mv(d, v.d, random=~1|country.diff)

summary(case2.summ.raw.diff)
summary(case2.summ.cohen.d)


############
# Part III #
############
m <- as.vector(by(dat$E, dat$CountryGroup, mean))
s <- as.vector(by(dat$E, dat$CountryGroup, sd))
n <- as.vector(by(dat$E, dat$CountryGroup, length))
group <- rep(sort(unique(dat$Group)), length(countries))
country <- rep(countries, each=4)
country.group <- 1:length(m)

case3.indiv <- lmer(E~Group-1+(1|Country)+(1|CountryGroup), data=dat)
case3.summ <- rma.mv(m, s^2/n, mods=~group-1, random=list(~1|country,~1|country.group))

summary(case3.indiv)
summary(case3.summ)


###########
# Part IV #
###########

abs.lat2 <- abs.lat[country]

case4.indiv <- lmer(E~Group*abs.lat-1-abs.lat+(1|Country)+(1|CountryGroup), data=dat)
case4.summ <- rma.mv(m, s^2/n, mods=~group*abs.lat2-1-abs.lat2, random=list(~1|country,~1|country.group))

summary(case4.indiv)
summary(case4.summ)


######################################
# Prepare individual data for Part V #
######################################
mv.dv <- c("N","E","O")
dat.long <- melt(dat[,mv.dv])
dat.long$id <- 1:nrow(dat)
dat.long$obs <- 1:nrow(dat.long)
dat.long$Country <- dat$Country
dat.long$Group <- dat$Group
dat.long$CountryGroup <- dat$CountryGroup
dat.long$DV.Group <- paste(dat.long$variable, dat.long$Group, sep="*")
dat.long$abs.lat <- abs.lat[as.character(dat.long$Country)]


###################################
# Prepare summary data for Part V #
###################################
nn <- table(dat$Group,dat$Country)
tmp1 <- by(dat[,mv.dv], dat$CountryGroup, colMeans)
tmp2 <- by(dat[,mv.dv], dat$CountryGroup, function(x){apply(x,2,var)})
tmp3 <- melt(sapply(tmp1, function(x){x}))
tmp3$v <- melt(sapply(tmp2, function(x){x}))$value / rep(as.vector(nn),each=length(tmp1[[1]]))
summ.m <- data.frame(
		DV=factor(mv.dv,levels=mv.dv), 
		Country=sapply(strsplit(as.character(tmp3[,2]), ": "), function(x){x[1]}),
		Group=sapply(strsplit(as.character(tmp3[,2]), ": "), function(x){x[2]}),
		CountryGroup=tmp3$Var2)
summ.m$DV.Group <- paste(summ.m$DV, summ.m$Group, sep="*")
summ.m$m <- tmp3$value
summ.m$abs.lat <- abs.lat[as.character(summ.m$Country)]


ugroup <- unique(summ.m$Group)
ucountry <- unique(summ.m$Country)
vcv.list <- list(NULL)
cnt <- 1
for(i in 1:length(ucountry)){
	for(j in 1:length(ugroup)){
		sel <- dat$Country==ucountry[i] & dat$Group==ugroup[j]
		tmp.vcv <- cov(dat[sel,mv.dv])
		vcv.list[[cnt]] <- tmp.vcv / nn[ugroup[j],ucountry[i]]
		cnt <- cnt + 1	
	}
}
summ.vcv <- matrixlist.to.matrix(vcv.list)
max(abs(tmp3$v - diag(summ.vcv)))
min(eigen(summ.vcv)$values)


##########
# Part V #
##########
case5.indiv <- lmer(value~DV.Group*abs.lat-1-abs.lat + (variable-1|Country) + (variable-1|CountryGroup), data=dat.long,
						control = lmerControl(optimizer = "optimx", calc.derivs = FALSE,
     			   		optCtrl = list(method = "nlminb", starttests = FALSE, kkt = FALSE)))
case5.summ <- rma.mv(m, summ.vcv, mods=~DV.Group*abs.lat-1-abs.lat,
	random=list(~DV|Country,~DV|CountryGroup),
	struct="UN", data=summ.m)

summary(case5.indiv)
summary(case5.summ)


rmse.mv <- as.data.frame(VarCorr(case5.indiv))[13,5]
tmpn <- rep(as.vector(nn), each=3)
case5.summ2 <- rma.mv(m, rmse.mv^2/tmpn, mods=~DV.Group*abs.lat-1-abs.lat,
	random=list(~DV|Country,~DV|CountryGroup),
	struct="UN", data=summ.m)

summary(case5.summ2)


# The lmer function from the lme4 package requires the error (level one) variance
# covariance matrix to be sigma^2 * Identity which is clearly unreasonable for this
# data as the three traits are likely correlated and have difference variances.
#
# The lme function from the older nlme packages allows for an unrestricted error 
# variance covariance and this yields results far closer to the summary data ones.
#
# The results are not identical because the summary data approach assumes this
# error variance covariance matrix is known and varies across country groups
# while the individual-level data approach assumes it is unknown and common
# across all individuals.
#
# Note: The lme function is slower and less stable than the lmer function and
# has more difficult syntax.
case5.indiv.lme <- lme(value~DV.Group*abs.lat-1-abs.lat, random=~(variable-1)|Country/CountryGroup, 
						correlation=corSymm(form=~1|Country/CountryGroup/id), 
						weights=varIdent(form=~1|variable), 
						control=lmeControl(maxIter=10000, msMaxIter=10000),
						data=dat.long)


# Note: The lme function can be used for the other case studies as well.
sel <- dat$Group=="Adult; Male"
case1.indiv.lme <- lme(E~1, random=~1|Country, data=dat[sel,])
sel <- dat$Group=="Adult; Male" | dat$Group=="Adult; Female"
case2.indiv.lme <- lme(E~Group-1, random=~1|Country/CountryGroup, data=dat[sel,])
case3.indiv.lme <-lme(E~Group-1, random=~1|Country/CountryGroup, data=dat)
case4.indiv.lme <-lme(E~Group*abs.lat-1, random=~1|Country/CountryGroup, data=dat)


save.image(file="supp.Rdata")
